#!/bin/csh


# this script creates 100 replicates of affected sib-pairs with
# their parents (see 1simped.dat). Two unlinked loci confer the susceptibility 
# of the disease.  The marginal penetrance at the first locus
# can be found in simdata_1.dat, while penetrances at the 
# second locus, conditional on the genotypes at the first
# (described in terms of one of 3 liability classes)
# can be found in simdata_2.dat 

unalias cp
unalias rm 

# I use the current time to set the seed

date +%s%N | cut -c 12-16 | awk '{print int( $1*0.3 )}'  > slinkin.dat
echo "100" >> slinkin.dat
echo "1" >> slinkin.dat
echo "0" >> slinkin.dat

# simulating data for the first locus

cp simdata_1.dat simdata.dat

cp 1simped.dat simped.dat

slink

# here I used the same datafile.dat for both loci

rm output1.out
sup -p pedfile.dat -d datafile.dat -o output1.out > /dev/null

# I use the pedfile.dat file that was just generated to create a new simped.dat.
# I add a liability column based on the genotypes at the first trait locus
# (the sum of the alleles minus 1), and I reset the genotypes at the descent
# locus to 0 0 for the non-founders.

awk '{if($3!=0 ){$13=0;$14=0}} {print $1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11+$12-1,"0 0",$13,$14,"2 Ped:",$1,"Ped:",$2}' pedfile.dat > simped.dat

# just 1 replicate of the 1000 families needs to be simulated

date +%s%N | cut -c 12-16 | awk '{print int( $1*0.3 )}'  > slinkin.dat
echo "1" >> slinkin.dat
echo "1" >> slinkin.dat
echo "0" >> slinkin.dat

cp simdata_2.dat simdata.dat

slink

rm output2.out
sup -p pedfile.dat -d datafile.dat -o output2.out > /dev/null




