#!/usr/bin/perl -w

# October 2006
# To find out where perl lives on your machine, type
#	which perl
# and adjust the first line of this script accordingly.
#
# This script assigns unique alleles to all founders.
# Input should be a file with the following fields (tab or space-delimited):
# 	PedigreeID IndividualID Father Mother Sex Dx (Liability)
# Output is written to file simpre.dat.
#
# The maximum number of pseudomarker alleles required is written to the 
# standard output.

use strict;
use Getopt::Std qw/ getopts /;

our ($opt_h, $opt_c, $opt_o, $opt_x);

sub init() {
  my $opt_string = 'hxc:o:';
  getopts("$opt_string") or usage();
  usage() if $opt_h;
  usage() unless @ARGV;
}

sub usage() {
  print STDERR << "EOF";

Usage:  $0 [-h] [-x] [-c code] [-o file] <prefile>
	-h          : show this help message
	-x          : X-linked markers
	-c string   : availability code
	-o string   : output file name

Defaults:  $0 -c2 -o simpre.dat <prefile>

EOF
	exit;
}

######################################################################
init();
$opt_c = 2 unless $opt_c;
$opt_o = "simpre.dat" unless $opt_o;

rename $opt_o, $opt_o . "_old" if -e $opt_o;
open OUT, ">$opt_o" or die "Cannot create $opt_o: $!";

my %uniq;                                     #hash to hold unique pedigree IDs
my $max=0;                    #max. number of alleles required for any pedigree

while (<>) {
  chomp;
  my ($ped,$id,$fa,$mo,$sex,@rest) = split;
  $uniq{$ped} = 1 unless $uniq{$ped};                          #new pedigree ID
  if ($fa == 0) {                                            #this is a founder
	if ($opt_x && $sex == 1) {             #for males with X-linked markers
	  	print OUT "$ped $id $fa $mo $sex @rest 0 0 $uniq{$ped} $uniq{$ped} $opt_c\n";
	  	$max = $uniq{$ped} if $uniq{$ped} > $max;
	  	$uniq{$ped} += 1;
	} 
	else {
	  print OUT "$ped $id $fa $mo $sex @rest 0 0 $uniq{$ped} ", $uniq{$ped}+1," $opt_c\n";
	  $max = $uniq{$ped}+1 if $uniq{$ped}+1 > $max;
	  $uniq{$ped} += 2;
	}
  }                                                         #end founder output
  else { print OUT "$ped $id $fa $mo $sex @rest 0 0 0 0 $opt_c\n"; }
}

close OUT;

print "The maximum number of pseudomarker alleles is $max.\n"
