/*Copyright 2006 Mathieu Lemire*/

#define SUPVER "061020"

#define DEBUG_M 0


#include "HaploDatafile.hh"
#include "Genotypes.hh"
#include "DescentPath.hh"
#include<ctime>


char *  pedfile;
char *  datfile; 
char *  outfile;
int  ncol, skip, nrepl;
long seed;
bool nodx=0, logfile=0, nodna=0;
bool prefile = 0;

void GetArg( int , char * [] );

main(int argc, char * argv[] ){


  cout << "SUP v" << SUPVER << endl;

  seed = time(0);  
  nrepl = 1;
  nodx=0; logfile=0;

  GetArg(argc, argv );
  char c;
  ifstream TEST (outfile, ios::in);
  if( TEST ){
    TEST.close();
    cout << outfile << " already exists. Overwrite [y/n] ? ";
    cin >> c;
    if( c != 'y' && c !='Y' ){
      exit(-1);
    }
  }

  ofstream OOO (outfile, ios::out);
  ofstream REC, HAP;

  if( logfile ){
    REC.open ("sup_recombination.log" , ios::out );
    HAP.open ("sup_haplotypes.log", ios::out); 
    REC << "FAMILY ID MEIOSIS INTERVAL" << endl;
    HAP << "FAMILY ID BLOCK HAPLOTYPE" << endl;
  }
 

  srand48(seed);

  cout << "Using seed: " << seed << endl;

  Prefile PF (pedfile, prefile, nodna);
  PF.FindParents();

 
  if( prefile )
    skip = PF.NColumns() - 9 - int(nodna); 
  else 
    skip = PF.NColumns() - 14;



  DescentPath DP  ( PF, skip  );

  
  
  HaploDatafile HD (datfile);

  bool sexlinked = HD.SexLinked();

  
  Genotypes G (HD.NLoci(), PF.NIndividuals() );


  int * h1,* h2;
  string foo;
  int locus = 0;

  int i,j,k,al1,al2, r;

  int p, l,u, tl = HD.TraitLocusPosition();
  bool allfilled;
  int ps1, ps2, ch0,ch1, tlch0, tlch1, fa, mo ;
  
  for( r = 0;  r< nrepl; r++){

    
  for(i=0; i<PF.NIndividuals(); i++){ 
    G.Filled(i, 0);
  }


  // segregation of the chromosomes

 
  for( p = 0; p<PF.NPedigrees() ; p++){
    
    //    cout << "Pedigree " << p << endl;
 
    l=PF.PedigreeBounds(p);
    u=PF.PedigreeBounds(p+1);
     
    G.Alloc( l, u );

    for(i=l; i<u;i++){ 
     
      // Filling the founders
      if( PF.IsFounder(i) ){
	locus = 0;
	for(j=0; j<HD.NHBlocks() ; j++){
	  
	  if( HD.InWhichBlock( HD.TraitLocusPosition() ) != j ){

	    h1 = HD.GenerateHaplotype( j );
	    if( sexlinked && PF.Sex(i)==1 ){
	      h2 = h1;
	    }else{
	      h2 = HD.GenerateHaplotype( j );
	    }
	   

	  } else {


	    h1 = HD.GenerateConditionalHaplotype( j,   
						  HD.LocusPositionInItsHaplotypeBlock(  HD.TraitLocusPosition() ),
						  DP.TraitLocusAlleles( i, 0 ) );
	    if(  sexlinked && PF.Sex(i)==1 ){
	      h2 = h1;
	    }else{
	      h2 = HD.GenerateConditionalHaplotype( j,   
						    HD.LocusPositionInItsHaplotypeBlock(  HD.TraitLocusPosition() ),
						    DP.TraitLocusAlleles( i, 1 ) );
	    }
	  }

	  if( logfile ){
	    HAP << r+1 << "_" << PF.PedigreeName(i) << " " << PF.IndividualName(i) << " " << j+1 << " ";  
	    for( k=0; k<HD.NumberOfSNPInBlock(j); k++){	  
	      HAP << h1[k];
	    } 
	    HAP << endl;
	    HAP << r+1 << "_" << PF.PedigreeName(i) << " " << PF.IndividualName(i) << " " << j+1 << " ";  
	    for( k=0; k<HD.NumberOfSNPInBlock(j); k++){	  
	      HAP << h2[k];
	    } 
	    HAP << endl;
	  }

	  for( k=0; k<HD.NumberOfSNPInBlock(j); k++){	  
	    
	    al1 = h1[k];
	    al2 = h2[k]; 
	    
	  
	    G.SetAllele( i, locus, 0, al1 );
	    G.SetAllele( i, locus, 1, al2 );
	  
	    locus++;
	  }
	}      
	G.Filled(i);				     
      }
    } 





    allfilled = 1;
    for( i=l; i<u && allfilled ;i++){
      if( !G.IsFilled(i) ) allfilled = 0;
    }

    while( !allfilled ){

      for( i=l; i<u ;i++){
	fa = PF.FatherIs( i );
	mo =  PF.MotherIs( i );
	// looking for an unfilled individual with filled parents
	if( !G.IsFilled(i) && G.IsFilled( fa ) &&  G.IsFilled( mo ) ){
	  
	  //cout << p << " " << i << " ... filling" << endl;

	  // those are the pseudomarker alleles
	  ps1 = DP.PseudoMarkerAlleles( i, 0 );
	  ps2 = DP.PseudoMarkerAlleles( i, 1 );
	  
	  // the pseudoalleles are not ordered according to the grandparetnatl origin,
	  // taking care of this first:
	  // ps1 should be received from the father

	  if( ps1 !=  DP.PseudoMarkerAlleles( fa, 0 ) &&  ps1 !=  DP.PseudoMarkerAlleles( fa, 1 ) ){
	    DP.SetPseudoMarkerAlleles( i, 0 , ps2 );
	    DP.SetPseudoMarkerAlleles( i, 1 , ps1 );
	    ps1 = DP.PseudoMarkerAlleles( i, 0 );
	    ps2 = DP.PseudoMarkerAlleles( i, 1 );
	  }


	  // segregation of the MAternal chromosome

	  if(  ps2 == DP.PseudoMarkerAlleles( mo, 0 ) ){
	    tlch1 = 0;
	  } else {
	    tlch1 = 1;
	  }

	  // starting at the trait locus;
	  
	  ch1 = tlch1;
	  al1 =  G.Alleles( mo, tl, ch1 );
	  G.SetAllele( i, tl, 1, al1 );

	  // segregation of the PAternal chromosome

	  if(  ps1 ==  DP.PseudoMarkerAlleles( fa, 0 ) ){
	    tlch0 = 0;
	  } else {
	    tlch0 = 1;
	  }

	  // starting at the trait locus;
	  
	  ch0 = tlch0;

	  if( sexlinked && PF.Sex(i)==1 ){
	    G.SetAllele( i, tl, 0, al1 );
	  } else {	  
	    G.SetAllele( i, tl, 0,  G.Alleles( fa, tl, ch0 ) );
	  }



	  for( locus = tl+1 ; locus < HD.NLoci() ; locus++){
	    // MAternal
	    
	    if( HD.Recombine( locus -1 , locus, 2 ) ){
	      if( logfile ) REC << r+1 << "_" << PF.PedigreeName(i) << " " << PF.IndividualName(i) << " Maternal " << locus << "-" << locus+1<< endl;
	      ch1 = 1-ch1;
	      }
	    al1 =  G.Alleles( mo, locus, ch1 );
	    G.SetAllele( i, locus, 1, al1 );	   


	    // PAternal
	    
	    if( HD.Recombine( locus -1 , locus, 1 ) ){
	      if( logfile ) REC << r+1 << "_" << PF.PedigreeName(i) << " " << PF.IndividualName(i) << " Paternal " << locus << "-" << locus+1<< endl;
	      ch0 = 1-ch0;
	    }

	    if( sexlinked && PF.Sex(i)==1 ){
	      // al1 is the one from above (Maternal)
	      G.SetAllele( i, locus, 0, al1 );
	    } else {
	      al1 =  G.Alleles( fa, locus, ch0 );
	      G.SetAllele( i, locus, 0,  al1 );	   
	    }

	  }

	  
	  ch0 = tlch0;
	  ch1 = tlch1;
	  for( locus = tl-1 ; locus >= 0 ; locus--){

	    if( HD.Recombine( locus, locus+1 , 2 ) ){
	      if( logfile ) REC <<r+1<<"_"<< PF.PedigreeName(i) << " " << PF.IndividualName(i) << " Maternal " << locus+1 << "-" << locus+2 << endl;
	      ch1 = 1 - ch1;
	      }
	    al1 =  G.Alleles( mo, locus, ch1 );
	    G.SetAllele( i, locus, 1, al1  );	   

	    if( HD.Recombine( locus, locus+1 , 1 ) ){
	      if( logfile ) REC << r+1 << "_"<< PF.PedigreeName(i) << " " << PF.IndividualName(i) << " Paternal " << locus+1 << "-" << locus +2<< endl;
	      ch0 = 1 - ch0;
	    }

	    if( sexlinked && PF.Sex(i)==1 ){
	      G.SetAllele( i, locus, 0, al1 );
	    } else {
	      al1 =  G.Alleles( fa, locus, ch0 );
	      G.SetAllele( i, locus, 0, al1  );	   
	    }

	    
	  

	  }
	      
	  G.Filled(i);

	}  
	 


      }

      
      allfilled = 1;
      for( i=l; i<u && allfilled ;i++){
	if( !G.IsFilled(i) ) allfilled = 0;
      }
    }

   
    for( i=l; i<u; i++){
      if( G.IsFilled(i) ){
      OOO << r +1 <<  "_" << PF.PedigreeName(i) << " " << PF.IndividualName(i)<<  " ";
      OOO << PF.FatherName(i) << " " << PF.MotherName(i) << " " << PF.Sex(i);

      istringstream R ( PF.RestOfLine(i) );

      for( j =0 ; j<skip; j++){
	R >> foo;
	OOO << " " << foo;
      }

      for( j=0; j< HD.NLoci() ; j++){	
	if( !nodx || j!=HD.TraitLocusPosition() ){ 
	  if( PF.HasDNA( i ) ){
	    OOO << "\t" <<  G.Alleles( i, j, 0 ) << " " << G.Alleles( i, j, 1 );
	  } else {
	    OOO << "\t" << "0 0";
	  }
	} 
      }
      OOO << endl;
    }
    }
   

    G.FreeMem( l,u );
 
 
  }
  
  if( !r ){
    cout << "Starting simulation." << endl;
    if( nrepl > 1 ){
      cout << "Done with 1st replicate" << endl;
   
    } 
  }
  
  if( nrepl>1 && ((r+1)% 500 == 0 || r+1==nrepl) ){ 
    cout <<"..." << r+1 << endl;cout.flush();
  } else if( (r+1)% 50 == 0 ){ 
    cout << "..." << r+1;
    cout.flush();
  }
 
  }

  if( nodx ){
    cout << "The trait locus is not present in the file " <<  outfile << "." << endl;
  } else {
    cout << "In file " << outfile << ", the trait locus genotypes are found in " << endl;
    cout << "tab-delimited column number " <<  HD.TraitLocusPosition()+2  << endl;   
  }

  if( logfile ){
    REC.close();
    HAP.close();
  }
  OOO.close();

  delete pedfile;
  delete datfile;
  delete outfile;

}




void GetArg( int argc, char * argv[] ){
  int i;
  bool found1=0, found2=0, found3=0;
  istringstream P;

 
  for(i=1;i<argc;i++){
 
    string arg ( argv[i] );
    if( arg.find("-p")==0 ){
      if( arg.find("-pr")==0 ){
	prefile = 1;
      } else {
	prefile = 0;
	nodna = 1;
      }
      if( i+1 < argc ){ 	
	string file1 ( argv[++i] );
	pedfile = new char [ file1.size()+1 ];
	file1.copy( pedfile , file1.size()  );
	pedfile[  file1.size() ] = '\0';	
	found1 = 1;
      } else {
	cout << "Missing argument for -p" << endl;
	exit(-1);
      }
    } else if( arg.find("-n")==0 ){
      nodx=1;
    } else if( arg.find("-e")==0 ){
      logfile=1;
    } else if(  arg.find("-d")==0 ){
      if( i+1 < argc ){
	string file2 ( argv[i+1] );

	datfile = new char [ file2.size()+1 ];
	file2.copy( datfile , file2.size()  );
	datfile[  file2.size() ] = '\0';
	i++;
	found2 = 1;
      } else {
	cout << "Missing argument for -d" << endl;
	exit(-1);
      }
    } else if( arg.find("-m")==0 ){
      nodna = 1;
    } else if(  arg.find("-r")==0 ){
      string R (argv[++i]);
      P.clear();
      P.str(R);
      P >> nrepl;
    } else if(  arg.find("-s")==0 ){
      string R (argv[++i]);
      P.clear();
      P.str(R);
      P >> seed;
    } else if(  arg.find("-o")==0 ){
      if( i+1 < argc ){
	string file3 ( argv[i+1] );
	
	outfile = new char [ file3.size()+1 ];
	file3.copy( outfile , file3.size()  );
	outfile[  file3.size() ] = '\0';
	i++;
	found3 = 1;
      } else {
	cout << "Missing argument for -o" << endl;
	exit(-1);
      }

    } else {
      cout << "Unrecognized option: " << arg << endl;
      exit(-1);
    }
    
  }

 
  if( !(found1*found2*found3) ){
    cout << "Missing arguments"<<endl; 
    cout << "Usage:\n  sup -p [-pre] <pedfile> -d <datafile> -o <outfile> [-s <seed=time(0)> -r <nrepl=1> -nd -e -m] " <<endl;
    exit(-1);
  }
 
}
